Rietveld Code Review Tool
Help | Bug tracker | Discussion group | Source code | Sign in
(28)

Unified Diff: src/pkg/io/io.go

Issue 4538088: code review 4538088: http,io,net: sendfile support (Closed)
Patch Set: diff -r 58102fac10c6 https://go.googlecode.com/hg Created 13 years, 10 months ago
Use n/p to move between diff chunks; N/P to move between comments. Please Sign in to add in-line comments.
Jump to:
View side-by-side diff with in-line comments
Download patch
Index: src/pkg/io/io.go
===================================================================
--- a/src/pkg/io/io.go
+++ b/src/pkg/io/io.go
@@ -117,6 +117,12 @@
ReadFrom(r Reader) (n int64, err os.Error)
}
+// ReaderNFrom is the interface that wraps the ReadNFrom method.
+type ReaderNFrom interface {
+ // ReadNFrom reads no more than n bytes from r.
+ ReadNFrom(r Reader, n int64) (int64, os.Error)
+}
+
// WriterTo is the interface that wraps the WriteTo method.
type WriterTo interface {
WriteTo(w Writer) (n int64, err os.Error)
@@ -216,16 +222,25 @@
// If dst implements the ReaderFrom interface,
// the copy is implemented by calling dst.ReadFrom(src).
r 2011/05/24 03:00:29 or ReadNFrom
func Copyn(dst Writer, src Reader, n int64) (written int64, err os.Error) {
- // If the writer has a ReadFrom method, use it to do the copy.
- // Avoids a buffer allocation and a copy.
- if rt, ok := dst.(ReaderFrom); ok {
+ // If the writer has a ReadNFrom or ReadFrom method, use it to
+ // do the copy. Avoids a buffer allocation and a copy.
+ hasReader := false
+ if rt, hasReader := dst.(ReaderNFrom); hasReader {
r 2011/05/24 03:00:29 this hasReader shadows the other one.
+ written, err = rt.ReadNFrom(src, n)
+ } else if rt, hasReader := dst.(ReaderFrom); hasReader {
written, err = rt.ReadFrom(LimitReader(src, n))
+ }
+ if hasReader {
if written < n && err == nil {
// rt stopped early; must have been EOF.
err = os.EOF
}
return
}
+ return BufferedCopyn(dst, src, n)
+}
+
+func BufferedCopyn(dst Writer, src Reader, n int64) (written int64, err os.Error) {
r 2011/05/24 03:00:29 doc comment. i'm not happy to see BufferedCopy ar
buf := make([]byte, 32*1024)
for written < n {
l := len(buf)
@@ -273,6 +288,10 @@
if wt, ok := src.(WriterTo); ok {
return wt.WriteTo(dst)
}
+ return BufferedCopy(dst, src)
+}
+
+func BufferedCopy(dst Writer, src Reader) (written int64, err os.Error) {
buf := make([]byte, 32*1024)
for {
nr, er := src.Read(buf)
« no previous file with comments | « src/pkg/http/server.go ('k') | src/pkg/net/Makefile » ('j') | src/pkg/net/sendfile_stub.go » ('J')

Powered by Google App Engine
RSS Feeds Recent Issues | This issue
This is Rietveld f62528b